// src/completion.builder.ts
import { json } from "@synstack/json";
import { pipe } from "@synstack/resolved";
import { t } from "@synstack/text";

// ../shared/src/ts.utils.ts
function never(neverCheck, value) {
  throw new Error(
    `Unexpected value, received: ${JSON.stringify(value ?? neverCheck, null, 2)}`
  );
}

// src/completion.runner.ts
import "@synstack/resolved";

// src/llm.types.ts
import "zod";

// src/completion.builder.ts
var CompletionBuilder2 = class _CompletionBuilder {
  _completion;
  constructor(completion2) {
    this._completion = completion2;
  }
  static get new() {
    return new _CompletionBuilder({});
  }
  static from(completion2) {
    return _CompletionBuilder.new.merge(completion2);
  }
  get $() {
    return this._completion;
  }
  $_(fn) {
    return pipe(this.$)._(fn);
  }
  async toSync() {
    return new _CompletionBuilder(await this.$);
  }
  /**
   * Override the values of the current builder with the provided values as object
   */
  merge(completion2) {
    return new _CompletionBuilder(
      pipe([this._completion, completion2])._(([old, added]) => ({
        ...old,
        ...added
      })).$
    );
  }
  // Todo: allow calling as a template directly
  /**
   * Set the system prompt
   */
  system(system) {
    return this.merge(
      pipe(system)._((system2) => ({
        system: system2
      })).$
    );
  }
  /**
   * Set the llm temperature
   * @argument temperature The temperature value between 0 and 1
   */
  temperature(temperature) {
    return this.merge(
      pipe(temperature)._((temperature2) => ({
        temperature: temperature2
      })).$
    );
  }
  /**
   * Set the maximum number of tokens to generate
   */
  maxTokens(maxTokens) {
    return this.merge(
      pipe(maxTokens)._((maxTokens2) => ({ maxTokens: maxTokens2 })).$
    );
  }
  /**
   * Set the stop sequences.
   *
   * If the LLM generates a stop sequence, the completion will stop.
   */
  stopSequences(sequences) {
    return this.merge(
      pipe(sequences)._((sequences2) => ({
        stopSequences: sequences2
      })).$
    );
  }
  /**
   * Set the top k tokens to consider for the completion
   */
  topK(topK) {
    return this.merge(
      pipe(topK)._((topK2) => ({
        topK: topK2
      })).$
    );
  }
  /**
   * Set the top p tokens to consider for the completion
   */
  topP(topP) {
    return this.merge(
      pipe(topP)._((topP2) => ({
        topP: topP2
      })).$
    );
  }
  /**
   * Reset the tools
   */
  clearTools() {
    return this.merge({
      toolsConfig: void 0
    });
  }
  /**
   * Set the tools available to the LLM
   *
   * @argument tools The tools to use
   * @argument requireToolUse Whether to require the LLM to use the tools
   */
  tools(tools, requireToolUse = false) {
    return this.merge(
      pipe(tools)._((tools2) => ({
        toolsConfig: {
          type: "multi",
          tools: tools2,
          requireToolUse
        }
      })).$
    );
  }
  /**
   * Set a single tool for the LLM to use. The LLM will always respond with this tool
   */
  tool(tool) {
    return this.merge(
      pipe(tool)._((tool2) => ({
        toolsConfig: {
          type: "single",
          tool: tool2
        }
      })).$
    );
  }
  /**
   * Set the stop reason for the completion
   */
  stopReason(stopReason) {
    return this.merge(
      pipe(stopReason)._((stopReason2) => ({
        stopReason: stopReason2
      })).$
    );
  }
  /**
   * Set the tokens usage
   */
  usage(usage) {
    return this.merge(
      pipe(usage)._((usage2) => ({
        usage: usage2
      })).$
    );
  }
  /**
   * Add usage to existing usage
   */
  addUsage(usage) {
    return this.merge(
      pipe([this.$, usage])._(([$, usage2]) => ({
        usage: {
          inputTokens: $.usage?.inputTokens ?? 0 + usage2.inputTokens,
          outputTokens: $.usage?.outputTokens ?? 0 + usage2.outputTokens
        }
      })).$
    );
  }
  /**
   * Set messages
   */
  messages(messages) {
    return this.merge(
      pipe(messages)._((messages2) => ({
        messages: messages2
      })).$
    );
  }
  /**
   * Add messages
   */
  addMessages(messages) {
    return this.merge(
      pipe([this.$, messages])._(([$, messages2]) => ({
        messages: [...$.messages ?? [], ...messages2]
      })).$
    );
  }
  /**
   * Add assistant message
   */
  addAssistantMessage(message) {
    return this.merge(
      pipe([this.$, message])._(([$, message2]) => {
        const $messages = $.messages ?? [];
        const lastMessage = $.messages.at(-1);
        if (!lastMessage)
          return {
            messages: [message2]
          };
        if (lastMessage.role !== "assistant")
          return {
            messages: [...$messages, message2]
          };
        return {
          messages: [
            ...$messages.slice(0, -1),
            {
              ...lastMessage,
              content: [...lastMessage.content, ...message2.content]
            }
          ]
        };
      }).$
    );
  }
  /**
   * Execute completion on Llm runner
   */
  run(runner) {
    return runner.runChatCompletion(this);
  }
  lastMessage() {
    return pipe(this.$)._((completion2) => {
      const lastMessage = completion2.messages?.at(-1);
      if (!lastMessage) throw new EmptyMessagesException(completion2);
      return lastMessage;
    }).$;
  }
  /**
   * Get last message as text
   */
  lastMessageText() {
    return pipe(this.lastMessage())._(
      (message) => message.content.filter((c) => c.type === "text").map((c) => c.text).join("")
    ).$;
  }
  /**
   * Get last message tool calls
   */
  lastMessageToolCalls() {
    return pipe(this.lastMessage())._((message) => {
      if (message.role !== "assistant") return [];
      return message.content.filter((c) => c.type === "tool_call");
    }).$;
  }
};
var EmptyMessagesException = class extends Error {
  constructor(completion2) {
    super(t`
      No messages in current completion state:
        ${json.serialize(completion2, { pretty: true })} 
    `);
  }
};
var completion = CompletionBuilder2.new;

// src/message.builder.ts
import { pipe as pipe2 } from "@synstack/resolved";

// ../fs/src/file.lib.ts
import { glob as glob2 } from "@synstack/glob";
import { json as json2 } from "@synstack/json";
import { path as path2 } from "@synstack/path";
import { Pipeable as Pipeable2 } from "@synstack/pipe";
import { str } from "@synstack/str";
import { xml } from "@synstack/xml";
import { yaml } from "@synstack/yaml";
import * as fsSync2 from "fs";
import * as fs2 from "fs/promises";
import "zod";

// ../fs/src/dir.lib.ts
import { git } from "@synstack/git";
import { glob } from "@synstack/glob";
import { path } from "@synstack/path";
import { Pipeable } from "@synstack/pipe";
import * as fsSync from "fs";
import * as fs from "fs/promises";

// ../fs/src/files-array.lib.ts
import { enhance } from "@synstack/enhance";
import "@synstack/path";
var filesArrayMethods = {
  filter(fn) {
    return files(this.$.filter(fn));
  },
  filterGlobs(...patterns) {
    return this.filter((file2) => file2.matchesGlobs(...patterns));
  },
  filterMimeTypes(...mimeTypes) {
    const types = new Set(mimeTypes);
    return this.filter((file2) => {
      const mimeType = file2.mimeType();
      if (mimeType === null) return false;
      return types.has(mimeType);
    });
  },
  filterDir(dirOrPath) {
    return this.filter((file2) => file2.isInDir(dirOrPath));
  },
  toPaths() {
    return this.map((file2) => file2.path);
  },
  relativePathsTo(dirOrFileOrPath) {
    return this.map((file2) => file2.relativePathTo(dirOrFileOrPath));
  }
};
var files = (files2) => enhance(
  "files_array",
  files2.map((f) => {
    if (f instanceof FsFile) return f;
    return file(f);
  }),
  filesArrayMethods
);
var filesFromDir = (dir2) => (files2) => enhance(
  "files_array",
  files2.map((f) => {
    if (f instanceof FsFile) return f;
    return dir2.file(f);
  }),
  filesArrayMethods
);

// ../fs/src/dir.lib.ts
var FsDir2 = class _FsDir extends Pipeable {
  _path;
  constructor(path3) {
    super();
    this._path = path3;
  }
  /**
   * Get the string representation of the directory path.
   *
   * @returns The absolute path of the directory as a string
   *
   * ```typescript
   * const srcDir = dir("./src");
   * console.log(srcDir.toString()); // "/absolute/path/to/src"
   * ```
   */
  toString() {
    return this._path;
  }
  /**
   * Get the primitive value of the directory path.
   *
   * @returns The absolute path of the directory
   */
  valueOf() {
    return this._path;
  }
  /**
   * Get the current instance of the directory.
   * Used for type compatibility with Pipeable.
   *
   * @returns The current FsDir instance
   */
  instanceOf() {
    return this;
  }
  /**
   * Get the absolute path of the directory.
   *
   * @returns The absolute path as a string
   *
   * ```typescript
   * const srcDir = dir("./src");
   * console.log(srcDir.path); // "/absolute/path/to/src"
   * ```
   */
  get path() {
    return this._path;
  }
  /**
   * Get the relative path from this directory to another file/directory
   *
   * @param dirOrfile - The other directory
   * @returns The relative path as a string
   */
  relativePathTo(dirOrfile) {
    return path.relative(this.path, dirOrfile.path);
  }
  /**
   * Get the relative path from another file/directory to this directory
   *
   * @param dirOrFile - The other directory
   * @returns The relative path as a string
   */
  relativePathFrom(dirOrFile) {
    if (dirOrFile instanceof FsFile)
      return this.relativePathFrom(dirOrFile.dir());
    return path.relative(dirOrFile.path, this.path);
  }
  /**
   * Create a new directory instance with the provided path.
   * Resolves relative paths to absolute paths.
   *
   * @param paths - One or more path segments to join into a directory path
   * @returns A new FsDir instance with the resolved path
   *
   * ```typescript
   * // Create from absolute path
   * const rootDir = dir("/path/to/project");
   *
   * // Create from relative path
   * const srcDir = dir("./src");
   *
   * // Create from multiple segments
   * const configDir = dir("project", "config");
   * ```
   */
  static cwd(...paths) {
    return new _FsDir(path.resolve(...paths));
  }
  /**
   * Create a new directory instance with a path relative to this directory.
   *
   * @param relativePath - The relative path to append to the current directory
   * @returns A new FsDir instance representing the combined path
   *
   * ```typescript
   * const projectDir = dir("/path/to/project");
   *
   * // Navigate to subdirectories
   * const srcDir = projectDir.to("src");
   * const testDir = projectDir.to("tests");
   *
   * // Navigate up and down
   * const siblingDir = srcDir.to("../other");
   * ```
   */
  to(relativePath) {
    const newPath = path.join(this._path, relativePath);
    return new _FsDir(newPath);
  }
  /**
   * Create a new file instance with a path relative to this directory.
   *
   * @param relativePath - The relative path to the file from this directory
   * @returns A new FsFile instance for the specified path
   * @throws If an absolute path is provided
   *
   * ```typescript
   * const srcDir = dir("./src");
   *
   * // Access files in the directory
   * const configFile = srcDir.file("config.json");
   * const deepFile = srcDir.file("components/Button.tsx");
   *
   * // Error: Cannot use absolute paths
   * srcDir.file("/absolute/path"); // throws Error
   * ```
   */
  file(relativePath) {
    if (path.isAbsolute(relativePath))
      throw new Error(`
Trying to access a dir file from an absolute paths:
  - Folder path: ${this._path}
  - File path: ${relativePath}
`);
    return FsFile.from(this._path, relativePath);
  }
  /**
   * Get the name of the directory (last segment of the path).
   *
   * @returns The directory name without the full path
   *
   * ```typescript
   * const srcDir = dir("/path/to/project/src");
   * console.log(srcDir.name()); // "src"
   * ```
   */
  name() {
    return path.filename(this._path);
  }
  /**
   * Check if the directory exists in the file system.
   *
   * @returns A promise that resolves to true if the directory exists, false otherwise
   *
   * ```typescript
   * const configDir = dir("./config");
   *
   * if (await configDir.exists()) {
   *   // Directory exists, safe to use
   *   const files = await configDir.glob("*.json");
   * } else {
   *   // Create the directory first
   *   await configDir.make();
   * }
   * ```
   */
  async exists() {
    return fs.access(this._path, fsSync.constants.F_OK).then(() => true).catch(() => false);
  }
  /**
   * Check if the directory exists in the file system synchronously.
   *
   * @synchronous
   * @returns True if the directory exists, false otherwise
   *
   * ```typescript
   * const configDir = dir("./config");
   *
   * if (configDir.existsSync()) {
   *   // Directory exists, safe to use
   *   const files = configDir.globSync("*.json");
   * } else {
   *   // Create the directory first
   *   configDir.makeSync();
   * }
   * ```
   */
  existsSync() {
    try {
      fsSync.accessSync(this._path, fsSync.constants.F_OK);
      return true;
    } catch (error) {
      if (error.code === "ENOENT") return false;
      throw error;
    }
  }
  /**
   * Create the directory and any necessary parent directories.
   *
   * @returns A promise that resolves when the directory is created
   *
   * ```typescript
   * const assetsDir = dir("./dist/assets/images");
   *
   * // Creates all necessary parent directories
   * await assetsDir.make();
   * ```
   */
  async make() {
    await fs.mkdir(this._path, { recursive: true });
  }
  /**
   * Create the directory and any necessary parent directories synchronously.
   *
   * @synchronous
   *
   * ```typescript
   * const assetsDir = dir("./dist/assets/images");
   *
   * // Creates all necessary parent directories immediately
   * assetsDir.makeSync();
   * ```
   */
  makeSync() {
    fsSync.mkdirSync(this._path, { recursive: true });
  }
  /**
   * Remove the directory and all its contents recursively.
   * If the directory doesn't exist, the operation is silently ignored.
   *
   * @returns A promise that resolves when the directory is removed
   *
   * ```typescript
   * const tempDir = dir("./temp");
   *
   * // Remove directory and all contents
   * await tempDir.rm();
   * ```
   */
  async rm() {
    await fs.rm(this._path, { recursive: true }).catch((e) => {
      if (e.code === "ENOENT") return;
      throw e;
    });
  }
  /**
   * Remove the directory and all its contents recursively synchronously.
   * If the directory doesn't exist, the operation is silently ignored.
   *
   * @synchronous
   *
   * ```typescript
   * const tempDir = dir("./temp");
   *
   * // Remove directory and all contents immediately
   * tempDir.rmSync();
   * ```
   */
  rmSync() {
    try {
      fsSync.rmSync(this._path, { recursive: true });
    } catch (error) {
      if (error.code === "ENOENT") return;
      throw error;
    }
  }
  /**
   * Find files in the directory that match the specified glob patterns.
   *
   * @param patterns - One or more glob patterns to match against
   * @returns A promise that resolves to an FsFileArray containing the matching files
   *
   * ```typescript
   * const srcDir = dir("./src");
   *
   * // Find all TypeScript files
   * const tsFiles = await srcDir.glob("**\/*.ts");
   *
   * // Find multiple file types
   * const assets = await srcDir.glob("**\/*.{png,jpg,svg}");
   *
   * // Use array of patterns
   * const configs = await srcDir.glob(["*.json", "*.yaml"]);
   * ```
   */
  async glob(...patterns) {
    return glob.cwd(this._path).find(...patterns).then(filesFromDir(this));
  }
  /**
   * Find files in the directory that match the specified glob patterns synchronously.
   *
   * @param patterns - One or more glob patterns to match against
   * @returns An FsFileArray containing the matching files
   * @synchronous
   *
   * ```typescript
   * const srcDir = dir("./src");
   *
   * // Find all TypeScript files synchronously
   * const tsFiles = srcDir.globSync("**\/*.ts");
   *
   * // Find multiple file types
   * const assets = srcDir.globSync("**\/*.{png,jpg,svg}");
   * ```
   */
  globSync(...patterns) {
    return filesFromDir(this)(glob.cwd(this._path).findSync(...patterns));
  }
  /**
   * Find files tracked by git in the directory.
   *
   * @returns A promise that resolves to an FsFileArray containing the git-tracked files
   *
   * ```typescript
   * const projectDir = dir("./project");
   *
   * // Get all git-tracked files in the directory
   * const trackedFiles = await projectDir.gitLs();
   * ```
   */
  async gitLs() {
    return git.ls(this._path).then(filesFromDir(this));
  }
};
var dir = FsDir2.cwd;

// ../fs/src/file.lib.ts
var FsFile = class _FsFile extends Pipeable2 {
  _path;
  _encoding;
  _schema;
  /**
   * Create a new FsFile instance from a path or a list of paths to be resolved.
   * The resulting path will be an absolute path.
   *
   * @param paths - One or more path segments to join into a file path
   * @returns A new FsFile instance with UTF-8 encoding
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const configFile = file("/path/to/config.json");
   * const relativeFile = file("./relative/path.txt");
   * const joinedFile = file("/base/path", "subdir", "file.txt");
   * ```
   */
  static from(...paths) {
    return new _FsFile(path2.join(...paths), "utf-8");
  }
  constructor(path3, encoding, schema) {
    super();
    this._path = path3;
    this._encoding = encoding ?? "utf-8";
    this._schema = schema ?? void 0;
  }
  /**
   * Provide a validation schema for the file content. To be used with:
   * - `.read.json`
   * - `.write.json`
   * - `.read.yaml`
   * - `.write.yaml`
   */
  /**
   * Provide a validation schema for JSON/YAML operations.
   * The schema will be used to validate data when reading or writing JSON/YAML files.
   *
   * @typeParam NewSchema - The Zod schema type for validation
   * @param schema - A Zod schema to validate JSON/YAML data
   * @returns A new FsFile instance with the schema attached
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   * import { z } from "zod";
   *
   * const ConfigSchema = z.object({
   *   port: z.number(),
   *   host: z.string()
   * });
   *
   * const config = await file("config.json")
   *   .schema(ConfigSchema)
   *   .read.json();
   * // config is typed as { port: number, host: string }
   * ```
   */
  schema(schema) {
    return new _FsFile(this._path, this._encoding, schema);
  }
  /**
   * Get the path of the file.
   *
   * @returns The absolute path of the file
   */
  valueOf() {
    return this._path;
  }
  /**
   * Get the current instance of the file.
   * Used for type compatibility with Pipeable.
   *
   * @returns The current FsFile instance
   */
  instanceOf() {
    return this;
  }
  // #region sub actions
  /**
   * Access the read operations for the file.
   * Provides methods for reading file contents in various formats.
   *
   * @returns An FsFileRead instance with methods for reading the file
   *
   * ```typescript
   * const content = await file("data.txt").read.text();
   * const json = await file("config.json").read.json();
   * const yaml = await file("config.yml").read.yaml();
   * ```
   */
  get read() {
    return new FsFileRead(
      this._path,
      this._encoding,
      this._schema
    );
  }
  /**
   * Access the write operations for the file.
   * Provides methods for writing content to the file in various formats.
   *
   * @returns An FsFileWrite instance with methods for writing to the file
   *
   * ```typescript
   * await file("data.txt").write.text("Hello");
   * await file("config.json").write.json({ hello: "world" });
   * await file("config.yml").write.yaml({ config: true });
   * ```
   */
  get write() {
    return new FsFileWrite(
      this._path,
      this._encoding,
      "overwrite",
      this._schema
    );
  }
  // #endregion
  // #region sync
  /**
   * Get the absolute path of the file.
   *
   * @returns The absolute path as a string
   */
  get path() {
    return this._path;
  }
  /**
   * Get the absolute path of the directory containing the file.
   *
   * @returns The directory path as a string
   */
  dirPath() {
    return path2.dirname(this._path);
  }
  /**
   * Get an FsDir instance representing the directory containing the file.
   *
   * @returns An FsDir instance for the parent directory
   *
   * ```typescript
   * const file = file("/path/to/file.txt");
   * const parentDir = file.dir(); // FsDir for "/path/to"
   * ```
   */
  dir() {
    return FsDir2.cwd(this.dirPath());
  }
  /**
   * Get the name of the file including its extension.
   *
   * @returns The file name with extension
   *
   * ```typescript
   * const file = file("/path/to/document.txt");
   * console.log(file.fileName()); // "document.txt"
   * ```
   */
  fileName() {
    return path2.filename(this._path);
  }
  /**
   * Get the extension of the file.
   *
   * @returns The file extension including the dot (e.g., ".txt")
   *
   * ```typescript
   * const file = file("/path/to/document.txt");
   * console.log(file.fileExtension()); // ".txt"
   * ```
   */
  fileExtension() {
    return path2.fileExtension(this._path);
  }
  /**
   * Get the name of the file without its extension.
   *
   * @returns The file name without extension
   *
   * ```typescript
   * const file = file("/path/to/document.txt");
   * console.log(file.fileNameWithoutExtension()); // "document"
   * ```
   */
  fileNameWithoutExtension() {
    return path2.filenameWithoutExtension(this._path);
  }
  /**
   * Get the MIME type of the file based on its extension.
   *
   * @returns The MIME type string or null if it cannot be determined
   *
   * ```typescript
   * const file = file("/path/to/image.png");
   * console.log(file.mimeType()); // "image/png"
   * ```
   */
  mimeType() {
    return path2.mimeType(this._path);
  }
  /**
   * Create a new FsFile instance with a path relative to this file's directory.
   *
   * @param relativePath - The relative path from this file's directory
   * @returns A new FsFile instance for the target path
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const sourceFile = file("/path/to/source.txt");
   * const targetFile = sourceFile.toFile("../output/target.txt");
   * // targetFile.path === "/path/output/target.txt"
   * ```
   */
  toFile(relativePath) {
    const newPath = path2.resolve(this.dirPath(), relativePath);
    return new _FsFile(newPath);
  }
  /**
   * Create a new FsDir instance with a path relative to this file's directory.
   *
   * @param relativePath - The relative path from this file's directory
   * @returns A new FsDir instance for the target directory
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const sourceFile = file("/path/to/source.txt");
   * const outputDir = sourceFile.toDir("../output");
   * // outputDir.path === "/path/output"
   * ```
   */
  toDir(relativePath) {
    const newPath = path2.join(this.dirPath(), relativePath);
    return FsDir2.cwd(newPath);
  }
  /**
   * Get the relative path from another file to this file
   *
   * ```ts
   * import { file } from "@synstack/fs";
   *
   * const file1 = file("/path/to/file1.txt");
   * const file2 = file("/path/to-other/file2.txt");
   *
   * console.log(file1.relativePathFrom(file2)); // ../to/file1.txt
   * ```
   */
  relativePathFrom(dirOrFile) {
    if (dirOrFile instanceof _FsFile)
      return this.relativePathFrom(dirOrFile.dir());
    return path2.relative(dirOrFile.path, this.path);
  }
  /**
   * Get the relative path to go from this file to another
   *
   * ```ts
   * import { file } from "@synstack/fs";
   *
   * const file1 = file("/path/to/file1.txt");
   * const file2 = file("/path/to-other/file2.txt");
   *
   * console.log(file1.relativePathTo(file2)); // ../to-other/file2.txt
   * ```
   */
  relativePathTo(dirOrFileOrPath) {
    return path2.relative(this.dirPath(), dirOrFileOrPath.path);
  }
  /**
   * Check if the file is located within the specified directory.
   *
   * @param dirOrPath - The directory or path to check against
   * @returns True if the file is in the directory, false otherwise
   *
   * ```typescript
   * import { file, dir } from "@synstack/fs";
   *
   * const sourceFile = file("/path/to/file.txt");
   * console.log(sourceFile.isInDir(dir("/path"))); // true
   * console.log(sourceFile.isInDir(dir("/other"))); // false
   * ```
   */
  isInDir(dirOrPath) {
    return path2.isInPath(dirOrPath.valueOf(), this._path.valueOf());
  }
  /**
   * Delete the file from the file system.
   * If the file doesn't exist, the operation is silently ignored.
   *
   * @returns A promise that resolves when the file is deleted
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const tempFile = file("./temp.txt");
   * await tempFile.write.text("temporary content");
   * await tempFile.rm(); // File is deleted
   * ```
   */
  async rm() {
    await fs2.rm(this._path, { recursive: true }).catch((e) => {
      if (e.code === "ENOENT") return;
      throw e;
    });
  }
  /**
   * Delete the file from the file system synchronously.
   * If the file doesn't exist, the operation is silently ignored.
   *
   * @synchronous
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const tempFile = file("./temp.txt");
   * tempFile.write.textSync("temporary content");
   * tempFile.rmSync(); // File is deleted immediately
   * ```
   */
  rmSync() {
    try {
      fsSync2.rmSync(this._path, { recursive: false });
    } catch (error) {
      if (error.code === "ENOENT") return;
      throw error;
    }
  }
  /**
   * Check if the file exists in the file system.
   *
   * @returns A promise that resolves to true if the file exists, false otherwise
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const configFile = file("./config.json");
   * if (await configFile.exists()) {
   *   const config = await configFile.read.json();
   * }
   * ```
   */
  async exists() {
    return fs2.access(this._path, fs2.constants.F_OK).then(() => true).catch(() => false);
  }
  /**
   * Check if the file exists in the file system synchronously.
   *
   * @synchronous
   * @returns True if the file exists, false otherwise
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const configFile = file("./config.json");
   * if (configFile.existsSync()) {
   *   const config = configFile.read.jsonSync();
   * }
   * ```
   */
  existsSync() {
    try {
      fsSync2.accessSync(this._path);
      return true;
    } catch (error) {
      if (error.code === "ENOENT") return false;
      throw error;
    }
  }
  /**
   * Get the creation date of the file.
   *
   * @returns A promise that resolves to the file's creation date
   * @throws If the file doesn't exist or cannot be accessed
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const sourceFile = file("./source.txt");
   * const created = await sourceFile.creationDate();
   * console.log(`File created on: ${created.toISOString()}`);
   * ```
   */
  async creationDate() {
    const fileStats = await fs2.stat(this._path);
    return fileStats.birthtime;
  }
  /**
   * Get the creation date of the file synchronously.
   *
   * @synchronous
   * @returns The file's creation date
   * @throws If the file doesn't exist or cannot be accessed
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const sourceFile = file("./source.txt");
   * const created = sourceFile.creationDateSync();
   * console.log(`File created on: ${created.toISOString()}`);
   * ```
   */
  creationDateSync() {
    const stats = fsSync2.statSync(this._path);
    return stats.birthtime;
  }
  /**
   * Check if the file path matches any of the provided glob patterns.
   *
   * @param globs - One or more glob patterns to match against, either as separate arguments or an array
   * @returns True if the file matches any pattern, false otherwise
   *
   * ```typescript
   * import { file } from "@synstack/fs";
   *
   * const sourceFile = file("./src/components/Button.tsx");
   * console.log(sourceFile.matchesGlobs("**\/*.tsx")); // true
   * console.log(sourceFile.matchesGlobs(["*.css", "*.html"])); // false
   * console.log(sourceFile.matchesGlobs("**\/*.ts", "**\/*.tsx")); // true
   * ```
   */
  matchesGlobs(...globs) {
    return glob2.matches(this._path, ...globs);
  }
  /**
   * Capture parts of the file path using a glob pattern
   *
   * ```ts
   * import { file } from "@synstack/fs";
   *
   * const myFile = file("/my-domain/my-sub-domain/features/feature-name.controller.ts");
   * const res = myFile.globCapture("/(*)/(*)/features/(*).controller.ts");
   * if (!res) throw new Error("File doesn't match glob pattern");
   * console.log(res[1]); // my-domain
   * console.log(res[2]); // my-sub-domain
   * console.log(res[3]); // feature-name.controller.ts
   * ```
   */
  globCapture(globPattern) {
    return glob2.capture(globPattern, this._path);
  }
};
var FsFileRead = class {
  _path;
  _encoding;
  _schema;
  constructor(path3, encoding, schema) {
    this._path = path3;
    this._encoding = encoding;
    this._schema = schema;
  }
  get path() {
    return this._path;
  }
  // #region sync
  /**
   * Read the file contents as a string.
   *
   * @returns A promise that resolves to the file contents as a string
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const content = await file("data.txt").read.text();
   * console.log(content); // "Hello, World!"
   * ```
   */
  async text() {
    return fs2.readFile(this._path, this._encoding);
  }
  /**
   * Read the file contents as a string synchronously.
   *
   * @synchronous
   * @returns The file contents as a string
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const content = file("data.txt").read.textSync();
   * console.log(content); // "Hello, World!"
   * ```
   */
  textSync() {
    return fsSync2.readFileSync(this._path, this._encoding);
  }
  /**
   * Read the file contents and return a chainable string instance.
   * Used for further manipulation of the content using @synstack/str methods.
   *
   * @returns A promise that resolves to a chainable string instance
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const content = await file("data.txt").read.str();
   * const lines = content
   *   .split("\n")
   *   .filter((line) => line.trim().length > 0);
   * ```
   */
  async str() {
    return this.text().then(str);
  }
  /**
   * Read the file contents and return a chainable string instance synchronously.
   * Used for further manipulation of the content using @synstack/str methods.
   *
   * @synchronous
   * @returns A chainable string instance
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const content = file("data.txt").read.strSync();
   * const lines = content
   *   .split("\n")
   *   .filter((line) => line.trim().length > 0);
   * ```
   */
  strSync() {
    return str(this.textSync());
  }
  /**
   * Read and parse the file contents as JSON.
   * If a schema is provided, the parsed data will be validated against it.
   *
   * @returns A promise that resolves to the parsed JSON data
   * @throws If the file doesn't exist, cannot be read, or contains invalid JSON
   * @throws If schema validation fails when a schema is provided
   *
   * ```typescript
   * interface Config {
   *   port: number;
   *   host: string;
   * }
   *
   * const config = await file("config.json")
   *   .schema(ConfigSchema)
   *   .read.json<Config>();
   * ```
   */
  async json() {
    return this.text().then(
      (t3) => json2.deserialize(t3, { schema: this._schema })
    );
  }
  /**
   * Read and parse the file contents as JSON synchronously.
   * If a schema is provided, the parsed data will be validated against it.
   *
   * @synchronous
   * @returns The parsed JSON data
   * @throws If the file doesn't exist, cannot be read, or contains invalid JSON
   * @throws If schema validation fails when a schema is provided
   *
   * ```typescript
   * const config = file("config.json")
   *   .schema(ConfigSchema)
   *   .read.jsonSync<Config>();
   * ```
   */
  jsonSync() {
    return json2.deserialize(this.textSync(), { schema: this._schema });
  }
  /**
   * Read and parse the file contents as YAML.
   * If a schema is provided, the parsed data will be validated against it.
   *
   * @typeParam T - The type of the parsed YAML data
   * @returns A promise that resolves to the parsed YAML data
   * @throws If the file doesn't exist, cannot be read, or contains invalid YAML
   * @throws If schema validation fails when a schema is provided
   *
   * ```typescript
   * interface Config {
   *   environment: string;
   *   settings: Record<string, unknown>;
   * }
   *
   * const config = await file("config.yml")
   *   .schema(ConfigSchema)
   *   .read.yaml<Config>();
   * ```
   */
  async yaml() {
    return this.text().then(
      (t3) => yaml.deserialize(t3, { schema: this._schema })
    );
  }
  /**
   * Read and parse the file contents as YAML synchronously.
   * If a schema is provided, the parsed data will be validated against it.
   *
   * @typeParam T - The type of the parsed YAML data
   * @synchronous
   * @returns The parsed YAML data
   * @throws If the file doesn't exist, cannot be read, or contains invalid YAML
   * @throws If schema validation fails when a schema is provided
   *
   * ```typescript
   * const config = file("config.yml")
   *   .schema(ConfigSchema)
   *   .read.yamlSync<Config>();
   * ```
   */
  yamlSync() {
    return yaml.deserialize(this.textSync(), { schema: this._schema });
  }
  /**
   * Read and parse the file contents as XML using @synstack/xml.
   * This parser is specifically designed for LLM-related XML processing.
   *
   * @typeParam T - The type of the parsed XML nodes array, must extend Array<Xml.Node>
   * @returns A promise that resolves to the parsed XML nodes
   * @throws If the file doesn't exist, cannot be read, or contains invalid XML
   * @see {@link https://github.com/pAIrprogio/synscript/tree/main/packages/xml|@synstack/xml documentation}
   *
   * ```typescript
   * interface XmlNode {
   *   tag: string;
   *   attributes: Record<string, string>;
   *   children: Array<XmlNode>;
   * }
   *
   * const nodes = await file("data.xml").read.xml<XmlNode[]>();
   * console.log(nodes[0].tag); // "root"
   * console.log(nodes[0].attributes.id); // "main"
   * ```
   *
   * @remarks
   * - Uses a non-spec-compliant XML parser tailored for LLM use cases
   * - Optimized for simple XML structures commonly used in LLM responses
   * - Does not support all XML features (see documentation for details)
   */
  async xml() {
    return this.text().then(xml.parse);
  }
  /**
   * Read and parse the file contents as XML synchronously using @synstack/xml.
   * This parser is specifically designed for LLM-related XML processing.
   *
   * @typeParam T - The type of the parsed XML nodes array, must extend Array<Xml.Node>
   * @synchronous
   * @returns The parsed XML nodes
   * @throws If the file doesn't exist, cannot be read, or contains invalid XML
   * @see {@link https://github.com/pAIrprogio/synscript/tree/main/packages/xml|@synstack/xml documentation}
   *
   * ```typescript
   * const nodes = file("data.xml").read.xmlSync<XmlNode[]>();
   * console.log(nodes[0].tag); // "root"
   * console.log(nodes[0].attributes.id); // "main"
   * ```
   *
   * @remarks
   * - Uses a non-spec-compliant XML parser tailored for LLM use cases
   * - Optimized for simple XML structures commonly used in LLM responses
   * - Does not support all XML features (see documentation for details)
   */
  xmlSync() {
    return xml.parse(this.textSync());
  }
  /**
   * Read the file contents and encode them as a base64 string.
   * Useful for handling binary data or preparing content for data URLs.
   *
   * @returns A promise that resolves to the file contents as a base64-encoded string
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * // Read and encode an image file
   * const imageBase64 = await file("image.png").read.base64();
   * console.log(imageBase64); // "iVBORw0KGgoAAAANSUhEUgAA..."
   *
   * // Create a data URL for use in HTML/CSS
   * const dataUrl = `data:image/png;base64,${imageBase64}`;
   * ```
   */
  async base64() {
    return fs2.readFile(this._path, "base64");
  }
  /**
   * Read the file contents and encode them as a base64 string synchronously.
   * Useful for handling binary data or preparing content for data URLs.
   *
   * @synchronous
   * @returns The file contents as a base64-encoded string
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const imageBase64 = file("image.png").read.base64Sync();
   * const dataUrl = `data:image/png;base64,${imageBase64}`;
   * ```
   */
  base64Sync() {
    return fsSync2.readFileSync(this._path, "base64");
  }
  /**
   * Read the file contents and create a synstack-compatible Base64Data object.
   * This format includes MIME type information along with the base64-encoded data.
   *
   * @param defaultMimeType - The MIME type to use if it cannot be determined from the file extension
   * @returns A promise that resolves to a Base64Data object containing the encoded content and MIME type
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * // Read an image with automatic MIME type detection
   * const imageData = await file("image.png").read.base64Data();
   * console.log(imageData);
   * // {
   * //   type: "base64",
   * //   data: "iVBORw0KGgoAAAANSUhEUgAA...",
   * //   mimeType: "image/png"
   * // }
   *
   * // Specify a custom MIME type for a binary file
   * const data = await file("custom.bin")
   *   .read.base64Data("application/custom");
   * ```
   */
  async base64Data(defaultMimeType = "application/octet-stream") {
    return {
      type: "base64",
      data: await this.base64(),
      mimeType: defaultMimeType
    };
  }
  /**
   * Read the file contents and create a synstack-compatible Base64Data object synchronously.
   * This format includes MIME type information along with the base64-encoded data.
   *
   * @param defaultMimeType - The MIME type to use if it cannot be determined from the file extension
   * @synchronous
   * @returns A Base64Data object containing the encoded content and MIME type
   * @throws If the file doesn't exist or cannot be read
   *
   * ```typescript
   * const imageData = file("image.png").read.base64DataSync();
   * console.log(imageData);
   * // {
   * //   type: "base64",
   * //   data: "iVBORw0KGgoAAAANSUhEUgAA...",
   * //   mimeType: "image/png"
   * // }
   * ```
   */
  base64DataSync(defaultMimeType = "application/octet-stream") {
    return {
      type: "base64",
      data: this.base64Sync(),
      mimeType: defaultMimeType
    };
  }
};
var FsFileWrite = class _FsFileWrite {
  _path;
  _encoding;
  _schema;
  _mode;
  constructor(path3, encoding, mode = "overwrite", schema) {
    this._path = path3;
    this._encoding = encoding;
    this._schema = schema;
    this._mode = mode;
  }
  /**
   * Set the write mode of the file
   * @argument preserve: If the file already exists, it will be left untouched
   * @argument overwrite: If the file already exists, it will be overwritten
   */
  mode(writeMode) {
    return new _FsFileWrite(this._path, this._encoding, writeMode, this._schema);
  }
  /**
   * Write text content to a file asynchronously.
   * Creates parent directories if they don't exist.
   * Respects the write mode (overwrite/preserve) setting.
   *
   * @param content - The content to write, will be converted to string using toString()
   * @returns A promise that resolves when the write operation is complete
   * @throws If the write operation fails or if parent directory creation fails
   */
  async text(content) {
    if (this._mode === "preserve" && await FsFile.from(this._path).exists())
      return;
    const dirname = path2.dirname(this._path);
    await fs2.mkdir(dirname, { recursive: true });
    await fs2.writeFile(this._path, content.toString(), this._encoding);
  }
  /**
   * Write text content to a file synchronously.
   * Creates parent directories if they don't exist.
   * Respects the write mode (overwrite/preserve) setting.
   *
   * @param content - The content to write, will be converted to string using toString()
   * @synchronous
   * @throws If the write operation fails or if parent directory creation fails
   */
  textSync(content) {
    if (this._mode === "preserve" && FsFile.from(this._path).existsSync())
      return;
    const dirname = path2.dirname(this._path);
    fsSync2.mkdirSync(dirname, { recursive: true });
    fsSync2.writeFileSync(this._path, content.toString(), this._encoding);
  }
  /**
   * Write data as JSON to a file asynchronously.
   * The data will be serialized using JSON.stringify.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be JSON-serializable
   * @returns A promise that resolves when the write operation is complete
   * @throws If schema validation fails or if the write operation fails
   */
  async json(data) {
    return this.text(json2.serialize(data, { schema: this._schema }));
  }
  /**
   * Write data as formatted JSON to a file asynchronously.
   * The data will be serialized using JSON.stringify with pretty printing.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be JSON-serializable
   * @returns A promise that resolves when the write operation is complete
   * @throws If schema validation fails or if the write operation fails
   */
  async prettyJson(data) {
    return this.text(
      json2.serialize(data, { schema: this._schema, pretty: true }) + "\n"
    );
  }
  /**
   * Write data as JSON to a file synchronously.
   * The data will be serialized using JSON.stringify.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be JSON-serializable
   * @synchronous
   * @throws If schema validation fails or if the write operation fails
   */
  jsonSync(data) {
    return this.textSync(json2.serialize(data, { schema: this._schema }));
  }
  /**
   * Write data as formatted JSON to a file synchronously.
   * The data will be serialized using JSON.stringify with pretty printing.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be JSON-serializable
   * @synchronous
   * @throws If schema validation fails or if the write operation fails
   */
  prettyJsonSync(data) {
    return this.textSync(
      json2.serialize(data, { schema: this._schema, pretty: true }) + "\n"
    );
  }
  /**
   * Write data as YAML to a file asynchronously.
   * The data will be serialized using YAML.stringify.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be YAML-serializable
   * @returns A promise that resolves when the write operation is complete
   * @throws If schema validation fails or if the write operation fails
   */
  async yaml(data) {
    return this.text(yaml.serialize(data, { schema: this._schema }));
  }
  /**
   * Write data as YAML to a file synchronously.
   * The data will be serialized using YAML.stringify.
   * If a schema is provided, the data will be validated before writing.
   *
   * @typeParam T - The type of data being written
   * @param data - The data to write, must be YAML-serializable
   * @synchronous
   * @throws If schema validation fails or if the write operation fails
   */
  yamlSync(data) {
    return this.textSync(yaml.serialize(data, { schema: this._schema }));
  }
  /**
   * Write base64-encoded data to a file asynchronously.
   * Creates parent directories if they don't exist.
   * Respects the write mode (overwrite/preserve) setting.
   *
   * @param data - The base64-encoded string to write
   * @returns A promise that resolves when the write operation is complete
   * @throws If the write operation fails or if parent directory creation fails
   */
  async base64(data) {
    if (this._mode === "preserve" && await FsFile.from(this._path).exists())
      return;
    const dirname = path2.dirname(this._path);
    await fs2.mkdir(dirname, { recursive: true });
    await fs2.writeFile(this._path, data.toString(), "base64");
  }
  /**
   * Write base64-encoded data to a file synchronously.
   * Creates parent directories if they don't exist.
   * Respects the write mode (overwrite/preserve) setting.
   *
   * @param data - The base64-encoded string to write
   * @synchronous
   * @throws If the write operation fails or if parent directory creation fails
   */
  base64Sync(data) {
    if (this._mode === "preserve" && FsFile.from(this._path).existsSync())
      return;
    const dirname = path2.dirname(this._path);
    fsSync2.mkdirSync(dirname, { recursive: true });
    return fsSync2.writeFileSync(this._path, data.toString(), "base64");
  }
};
var file = FsFile.from;

// src/message.builder.ts
import { t as t2, tParse } from "@synstack/text";
var userMsg = (template, ...values) => pipe2(t2(template, ...values))._(tParse)._(
  (parts) => parts.map((v) => {
    if (typeof v === "string")
      return { type: "text", text: v };
    if (v.type === "base64")
      return {
        type: "image",
        image: v
      };
    if (v.type === "tool_response") return v;
    never(v);
  })
)._(
  (content) => ({
    role: "user",
    content
  })
).$;
var assistantMsg = (template, ...values) => {
  return pipe2(t2(template, ...values))._(tParse)._(
    (parts) => parts.map((v) => {
      if (typeof v === "string")
        return { type: "text", text: v };
      if (v.type === "tool_call") return v;
      never(v.type);
    })
  )._(
    (content) => ({
      role: "assistant",
      content
    })
  ).$;
};
export {
  CompletionBuilder2 as CompletionBuilder,
  assistantMsg,
  completion,
  userMsg
};
//# sourceMappingURL=llm.index.js.map